/*
	*
	* This file is taken from CoreStats <https://gitlab.com/cubocore/corestats>
	* Further modifications may be done as per the requirements of DesQ
	* All bug repots are to be filed with DesQ and not corestats
	*

	*	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, *
	MA 02110-1301, USA.
	*
*/

#include <desq/DesQUtils.hpp>
#include "DesQUsageGauge.hpp"

static QList<QColor> colors = {
	0x1ABC9C, 0x9B59B6, 0xFF8F00, 0xE74C3C, 0x4E342E,
	0x8E44AD, 0xEF6C00, 0xC0392B, 0x34495E, 0x424242,
	0x5499C7, 0xCD6155, 0xF5B041, 0x566573, 0x58D68D
};

DesQUsageGauge::DesQUsageGauge( QString resource, int segments, QWidget *parent ) : QWidget( parent ) {

	mSegments = segments > 5 ? 5 : segments;
	if ( segments > 5 )
		qWarning() << "DesQUsageGauge for" << resource << "will show only 5 indicators instead of" << segments;

	mUnits = DesQUsageGauge::PERCENT;
	resName = QString( resource );

	setMinimumSize( QSize( 100, 100 ) );

	setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );

	for ( int i = 0; i < segments; i++ ) {
		mValues.append( 0 );
	}
}

void DesQUsageGauge::setMax( quint64 max ) {

	mMax = max;

	repaint();
}

void DesQUsageGauge::setUnits( DesQUsageGauge::Units units ) {

	mUnits = units;
	repaint();
}

void DesQUsageGauge::setLabels( QStringList labels ) {

	mLabels.clear();
	mLabels << labels;

	QFontMetrics fm( font() );
	labelHeight = fm.lineSpacing() * labels.count() + ( labels.count() - 1 ) * 5;

	repaint();
};

void DesQUsageGauge::setValues( GaugeResourceValues values ) {

	mValues.clear();
	mValues << values;

	repaint();
}

QSize DesQUsageGauge::sizeHint() {

	int x = qMin( width(), height() );
	return QSize( x, x );
}

QSize DesQUsageGauge::minimumSizeHint() {

	return QSize( minWidth, minWidth );
}

int DesQUsageGauge::minimumWidth() {

	return minWidth;
}

void DesQUsageGauge::paintEvent( QPaintEvent *pEvent ) {

	int side = qMin( width() - 50 - 10, height() - 10 );
	QImage img( side, side, QImage::Format_ARGB32 );
	img.fill( Qt::transparent );

	QPainter painter( &img );
	painter.setRenderHints( QPainter::Antialiasing | QPainter::TextAntialiasing );

	painter.save();
	QPalette pltt = qApp->palette();
	QColor bg = pltt.color( QPalette::Highlight );
	bg.setAlpha( 27 );
	painter.setPen( Qt::NoPen );
	painter.setBrush( bg );
	painter.drawEllipse( QPointF( side / 2, side / 2 ), side / 2 - 1, side / 2 - 1 );
	painter.restore();

	/* Start at 12 o'clock */
	int startAngle = -135 * 16;

	double penWidth = 5.0;

	/* Draw the finish line */
	painter.save();
	QLineF angleLine;
	angleLine.setP1( QPointF( side / 2, side / 2 ) );
	angleLine.setAngle( -45.0 );
	angleLine.setLength( 20 );
	angleLine.translate( side / 2 - 40, side / 2 - 40 );
	painter.setPen( QPen( pltt.color( QPalette::Highlight ), 2.0, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin ) );
	painter.drawLine( angleLine );
	painter.restore();

	int x = 2;
	int y = 2;
	for ( int s = 0; s < mSegments; s++ ) {
		painter.save();

		painter.setPen( QPen( colors.at( s ), penWidth, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin ) );
		double spanAngle = mValues.value( s ) * -270.0 * 16.0 / mMax;

		x += penWidth + 2;
		y += penWidth + 2;

		painter.drawArc( QRect( x, y, side - 2 * x, side - 2 * y ), startAngle, spanAngle );
		painter.restore();
	}

	QString text;
	switch( mUnits ) {
		case DesQUsageGauge::PERCENT: {
			text = "%";
			break;
		}

		case DesQUsageGauge::SIZE: {
			text = DesQUtils::formatSizeStr( mValues.value( 0 ) );
			break;
		}

		case DesQUsageGauge::SPEED: {
			text = DesQUtils::formatSizeStr( mValues.value( 0 ) ) + "/s";
			break;
		}
	}

	painter.setPen( Qt::NoPen );
	painter.setBrush( pltt.color( QPalette::HighlightedText ).darker() );
	painter.drawEllipse( QPoint( side / 2, side / 2 ), 25, 25 );

	painter.save();
	painter.setPen( pltt.color( QPalette::HighlightedText ) );
	painter.setFont( QFont( font().family(), 8 ) );
	painter.drawText( QRectF( 0, 0, side, side ), Qt::AlignCenter, resName + "\n" + text );
	painter.restore();
	painter.end();

	painter.begin( this );
	painter.setRenderHints( QPainter::Antialiasing );
	painter.fillRect( rect(), QColor( 0, 0, 0, 10 ) );
	painter.fillRect( rect(), QColor( 255, 255, 255, 10 ) );
	painter.drawImage( QRect( 5, ( height() - side ) / 2, side, side ), img );

	painter.save();
	y = ( height() - labelHeight ) / 2 - 5;			// We will be adding 5, so reduce that amount at first.
	y = ( y > 0 ? y : 0 );
	QRectF bound( 0, y, 0, 0 );
	for( int s = 0; s < mLabels.count(); s++ ) {
		painter.setPen( colors.value( s ) );
		painter.drawText( QRect( side + 5, bound.bottom() + 5, 45, height() ), Qt::AlignTop | Qt::AlignRight, mLabels.value( s ), &bound );
	}
	painter.restore();

	painter.end();

	pEvent->accept();
};
